﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Hotline.Identity.Roles;
using Hotline.Orders;
using Hotline.Repository.SqlSugar.DataPermissions;
using Hotline.Repository.SqlSugar.Extensions;
using Hotline.Settings.TimeLimits;
using Hotline.Share.Consts;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Enums.Order;
using SqlSugar;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;

namespace Hotline.Repository.SqlSugar.Orders
{
    public class OrderRepository : BaseRepositoryWorkflow<Order>, IOrderRepository, IScopeDependency
    {
        private readonly IDataPermissionFilterBuilder _dataPermissionFilterBuilder;
        private readonly ITimeLimitDomainService _timeLimitDomainService;

		public OrderRepository(ISugarUnitOfWork<HotlineDbContext> uow, IDataPermissionFilterBuilder dataPermissionFilterBuilder, ITimeLimitDomainService timeLimitDomainService) : base(uow, dataPermissionFilterBuilder)
        {
            _dataPermissionFilterBuilder = dataPermissionFilterBuilder;
            _timeLimitDomainService = timeLimitDomainService;

		}

        public async Task<string> AddOrderNavAsync(Order order, CancellationToken cancellationToken)
        {
            if (order.OrderType == EOrderType.MarketSupervisionBy12315)
            {
                //弥补AddNav方法没有自动指派到创建人
                order.AssignToCreator(_dataPermissionFilterBuilder.DataPermissionManager);

                if (order.AcceptTypeCode == AcceptTypeConst.TouSu)
                {
                    order.OrderComplain.InitDatePermission(_dataPermissionFilterBuilder.DataPermissionManager);
                    await AddNav(order).Include(d => d.OrderComplain).ExecuteCommandAsync();
                }
                else if (order.AcceptTypeCode == AcceptTypeConst.JuBao)
                {
                    order.OrderReport.InitDatePermission(_dataPermissionFilterBuilder.DataPermissionManager);
                    await AddNav(order).Include(d => d.OrderReport).ExecuteCommandAsync();
                }
                else
                {
                    throw UserFriendlyException.SameMessage("12315工单不支持该受理类型");
                }
            }
            else
            {
                await AddAsync(order, cancellationToken);
            }
            return order.Id;
        }

        public async Task UpdateOrderNavAsync(Order order, CancellationToken cancellationToken)
        {
            await UpdateAsync(order, cancellationToken);
            if (order.OrderType == EOrderType.MarketSupervisionBy12315)
            {
                if (order.AcceptTypeCode == AcceptTypeConst.TouSu)
                {
                    await Db.Insertable<OrderComplain>(order.OrderComplain).ExecuteCommandAsync();
                }
                else if (order.AcceptTypeCode == AcceptTypeConst.JuBao)
                {
                    await Db.Insertable<OrderReport>(order.OrderReport).ExecuteCommandAsync();
                }
                else
                {
                    throw UserFriendlyException.SameMessage("12315工单不支持该受理类型");
                }
            }
        }

        public async Task RemoveOrderComplainAsync(OrderComplain orderComplain, CancellationToken cancellationToken)
        {
            await Db.Deleteable(orderComplain).ExecuteCommandAsync();
        }

        public async Task RemoveOrderReportAsync(OrderReport orderReport, CancellationToken cancellationToken)
        {
            await Db.Deleteable(orderReport).ExecuteCommandAsync();
        }

        public async Task<List<AboutToExpireDto>> GetAboutToExpireAsync(AboutToExpireListDto dto,int AboutToExpireNum, CancellationToken cancellationToken)
        {
            DateTime stTime= DateTime.Now; 
            //判断是否在工作日
            if (_timeLimitDomainService.IsWorkDay(stTime))
            {
                    
            }
            //        var oders = await Db.Queryable<Order>()
            //            .WhereIF(dto.IsProvince.HasValue, x => x.IsProvince == dto.IsProvince)
            //            .WhereIF(!string.IsNullOrEmpty(dto.No), x => x.No == dto.No)
            //            .WhereIF(!string.IsNullOrEmpty(dto.Title), x => x.Title.Contains(dto.Title!))
            //            .Select(x => new AboutToExpireDto()
            //            {
            //             IntervalText = _timeLimitDomainService.CalcTimeInterval(DateTime.Now,x.)
            //}, true)

            return new List<AboutToExpireDto>();
        }

    }
}
